import weewx
from weewx.wxengine import StdService

class RetainLoopValues(StdService):
    """Service retains previous loop packet values updating any value that isn't None from new
    packets. It then replaces the original packet with a new packet that contains all of the values; 
    the original unmodified packet will be stored on the event in a property named 'originalPacket'."""

    def __init__(self, engine, config_dict):
        super(RetainLoopValues, self).__init__(engine, config_dict)
        self.bind(weewx.NEW_LOOP_PACKET, self.newLoopPacket)
        self.retainedLoopValues = {}

    def newLoopPacket(self, event):
        event.originalPacket = event.packet
        self.retainedLoopValues.update( dict((k,v) for k,v in event.packet.iteritems() if v is not None) )
        event.packet = self.retainedLoopValues.copy()
